; =================================================================================================
; Title:   MemLeak.asm
; Author:  G. Friedrich  @ March 2005
; Version: 1.0.0
; Purpose: Memory Leakage demonstration program.
; =================================================================================================


%include @Environ(OA32_PATH)\\Code\\Macros\\SetupSys.inc    ;Include & initialize standard modules
SetupSys OOP_WINDOWS, DEBUG(WND)                           ;Loads OOP files and OS related objects

include MemLeak_Globals.inc                                 ;Includes application globals
IncludeBoth ImageHlp

LoadObjects Dialog, DialogModal, DialogAbout                ;Loads or builds the following objects
LoadObjects DialogModeless
MakeObjects IAT32Hook
LoadObjects SimpleImageList, MaskedImageList
LoadObjects MsgInterceptor, WinControl, Toolbar, Rebar, Statusbar, Tooltip
LoadObjects SdiApp

.data
  pIAT32Hook_A  Pointer   NULL
  pIAT32Hook_F  Pointer   NULL
  pIAT32Hook_R  Pointer   NULL
  dAllocCount   dword     0
  dFreeCount    dword     0
  dReallocCount dword     0
    
.code
include MemLeak.inc                                         ;Includes MemLeak object

MyHeapAlloc proc hHeap:Handle, dFlags:dword, dBytes:dword
  push dBytes
  push dFlags
  push hHeap
  mov eax, pIAT32Hook_A
  call [eax].IAT32Hook.pEntry
  .if eax
    inc dAllocCount
  .endif
  ret
MyHeapAlloc endp

MyHeapFree proc hHeap:Handle, dFlags:dword, pMem:Pointer
  push pMem
  push dFlags
  push hHeap
  mov eax, pIAT32Hook_F
  call [eax].IAT32Hook.pEntry
  .if eax
    inc dFreeCount
  .endif
  ret
MyHeapFree endp

MyHeapRealloc proc hHeap:Handle, dFlags:dword, pMem:Pointer, dBytes:dword
  push dBytes
  push pMem
  push dFlags
  push hHeap
  mov eax, pIAT32Hook_R
  call [eax].IAT32Hook.pEntry
  .if eax
    inc dReallocCount
  .endif
  ret
MyHeapRealloc endp

pApp equ offset @MemLeak                                    ;Symbol containing the pointer to 
                                                            ;  the application object template
start:                                                      ;Program entry point
    mov hInstance, $invoke(GetModuleHandle, NULL)           ;Retrieves module instance handle
    ObjectsInit                                             ;Runtime initialization of OOP model
    mov pIAT32Hook_F, $New(IAT32Hook, Init, esi, "Kernel32.dll", "HeapFree" , offset MyHeapFree)
    mov pIAT32Hook_R, $New(IAT32Hook, Init, esi, "Kernel32.dll", "HeapReAlloc" , offset MyHeapRealloc)
    mov pIAT32Hook_A, $New(IAT32Hook, Init, esi, "Kernel32.dll", "HeapAlloc", offset MyHeapAlloc)
    OCall pApp::MemLeak.Init                                ;Initialize the object data
    OCall pApp::MemLeak.Run                                 ;Execute the application
    OCall pApp::MemLeak.Done                                ;Finalize it
    Destroy pIAT32Hook_F
    Destroy pIAT32Hook_R
    Destroy pIAT32Hook_A
    DbgDec dAllocCount
    DbgDec dFreeCount
    DbgDec dReallocCount
    ObjectsDone                                             ;Runtime finalization of the OOP model
    invoke ExitProcess, 0                                   ;Exits program returning 0 to the OS
end start                                                   ;Code end and defines prg entry point
